#ifndef __CAMERA3_FORMAT_CONVERT_HELPER__
#define __CAMERA3_FORMAT_CONVERT_HELPER__

#include <utils/Errors.h>
#include <hardware/gralloc.h>
#ifdef HAL3_HWC_BUFFER
#include <libRTKAllocator/include/hardware/GrallocWrapper.h>
#endif
#include "Base.h"
#include "Common.h"
//#include "CameraMonitor.h"

#ifdef HAL3_HWC_BUFFER
#define RTK_RAW_BUFFER(x)    (  x==HAL_PIXEL_FORMAT_RTK_16_YCbCr_420_SP\
                             || x==HAL_PIXEL_FORMAT_RTK_W16_H16_YCrCb_420_SP\
                             || x==HAL_PIXEL_FORMAT_RTK_W16_H16_YCbCr_420_SP \
                             )
#else
#include "V4L2Common.h"
#define RTK_RAW_BUFFER(x)    (  x==HAL_PIXEL_FORMAT_YCbCr_420_888 )
#endif

using namespace android;
// using namespace android_camera_hal3;

namespace androidcamera3
{

typedef struct _BUF_CONVERT_INFO {
    int32_t         w;
    int32_t         h;
    int32_t         format;
    void*            vAddr;
    uint32_t        phyAddr;
    int32_t         bufferSize;
    buffer_handle_t* pHandle;
#ifdef HAL3_HWC_BUFFER
    AllocatorDef*    allocator;
#endif
} BUF_CONVERT_INFO;

#ifdef HAL3_HWC_BUFFER
#else

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <libexif/exif-data.h>
#include <libexif/exif-content.h>
#include <libexif/exif-format.h>
#include <libexif/exif-mem.h>
#include <libexif/exif-loader.h>
#include <libexif/exif-tag.h>
#include <libexif/exif-entry.h>
extern "C" {
#include "jpeg-data.h"
#include "jpeg-marker.h"
};
/* this file will be unilaterally overwritten */
#define FILE_NAME "/tmp/write-exif.jpg"


/* length of data in image_jpg */
static const unsigned int image_jpg_len = 0; //= sizeof(image_jpg);

/* dimensions of image */
//static const unsigned int image_jpg_x = 64;
//static const unsigned int image_jpg_y = 64;

/* start of JPEG image data section */
static const unsigned int image_data_offset = 20;
//#define image_data_len (image_jpg_len - image_data_offset)

/* raw EXIF header data */
static const unsigned char exif_header[] = {
    0xff, 0xd8, 0xff, 0xe1
};
/* length of data in exif_header */
static const unsigned int exif_header_len = sizeof(exif_header);

/* byte order to use in the EXIF block */
#define FILE_BYTE_ORDER EXIF_BYTE_ORDER_INTEL

/* comment to write into the EXIF block */
#define FILE_COMMENT "libexif demonstration image"

/* special header required for EXIF_TAG_USER_COMMENT */
#define ASCII_COMMENT "ASCII\0\0\0"

/*struct GPSinfo {
    float latitude;
    float longitude;
    int altitude;
	long timestamp;
    const char *method;
};*/
#endif

class FormatConvertHelper
{

public:
    FormatConvertHelper(/*int32_t v4l2Fmt, int32_t halFmt*/
#ifdef HAL3_HWC_BUFFER
        GrallocWrapper* pGrallocWrapper
#endif
    );
    ~FormatConvertHelper();

    status_t convertBuffers(Request* r);
    status_t convertBuffers(Request* r, BUF_CONVERT_INFO* ext);
#ifdef HAL3_HWC_BUFFER
#else
    int saveThumbnail(unsigned char* inputBuffer, unsigned char* outputBuffer, int thumbWidth, int thumbHeight, int thumbQ);
    int appendThumb(const char* filename, const char* thumbFilename);
    void insert_thumb (ExifData* ed, ExifLog* log, const char* thumbnail);
    void saveJpeg (ExifData* ed, ExifLog* log, const char* filename);
    int Addexif(const char* filename, const char* thumbFilename, int image_jpg_x, int image_jpg_y, MISC_FIELD_INFO* info);
    void commitChanges(const char* filename);
    status_t convertBuffers(Request* r, struct vdIn* videoIn_F);
    int savePicture(unsigned char* inputBuffer, int width, int height, const char* filename, int jpegQ);
    int saveYUYVtoJPEG (unsigned char* inputBuffer, int width, int height, FILE* file, int jpegQ);
    void compressToJPEG(BUF_CONVERT_INFO* src, BUF_CONVERT_INFO* dst, MISC_FIELD_INFO* info);
#endif
    void setRtkPrivData(Request* r, uint32_t inputFps);

    void compressJPEG(
        JPEG_SRC_BUF_INFO* pSrc,
        JPEG_DST_BUF_INFO* pDst);

    bool getPrivData(StreamBuffer* pStreamBuf);

private:
    //int32_t                     mFormat;
    int32_t                     mHalPixelFormat;

    unsigned long               mSeHandle;

    alloc_device_t*             mBufAlignRemove_pAllocDev;
    const struct hw_module_t*   mGrallocModule;
#ifdef HAL3_HWC_BUFFER
    GrallocWrapper*             mGrallocWrapper;
#else
    struct vdIn*                 videoIn_F;
#endif
    uint32_t                    mJpegMetaDstRawCount;
    uint32_t                    mJpegMetaSrcRawCount;
    uint32_t                    mJpegRstCount;

    // backward compatible...
    uint8_t*                    mThumbBuffer;
    int32_t                     mThumbBufSize;

    // CameraMonitor *             mCameraMonitor;

    bool se_init(alloc_device_t** ppAllocDev);

    void se_uninit(alloc_device_t** ppAllocDev);

    bool ScaleYuvDstNoAlign(
        uint32_t srcPhyBuffer,
        uint8_t** ppYuv,
        uint32_t* scaled_buf_size,
        int iScaleWidthTo,
        int iScaleHeightTo,
        int srcWidth,
        int srcHeight);

    bool ScaleYuv(
        uint32_t srcPhyBuffer,
        uint8_t** ppYuv,
        uint32_t* scaled_buf_size,
        int iScaleWidthTo,
        int iScaleHeightTo,
        int srcWidth,
        int srcHeight);

    bool se_Create420spPhyBuffer(
        alloc_device_t* pAllocDev,
        int iWidth,
        int iHeight,
        int format,
        buffer_handle_t* pBufferHandle,
        uint32_t* pPhyBufferHandle);

    bool se_PhyBuf2VirBuf(
        int iWidth,
        int iHeight,
        buffer_handle_t pBufferHandle,
        void** pvirtaddr);

    bool se_DestroyPhyBuffer(
        alloc_device_t*  pAllocDev,
        buffer_handle_t pBufferHandle);

    bool isCompressedFormat(int v4l2Fmt);

    void findSrcBufInfo(
        const Request* r,
        BUF_CONVERT_INFO* pSrcInfo);

    status_t convertOneBuffer(
        BUF_CONVERT_INFO* src,
        BUF_CONVERT_INFO* dst,
        MISC_FIELD_INFO* info);

    status_t convertToRtkRawBuffer(
        BUF_CONVERT_INFO* src,
        BUF_CONVERT_INFO* dst);

    status_t convertToNativeRawBuffer(
        BUF_CONVERT_INFO* src,
        BUF_CONVERT_INFO* dst);

    status_t convertToJPEG(
        BUF_CONVERT_INFO* src,
        BUF_CONVERT_INFO* dst,
        MISC_FIELD_INFO* info);

    status_t processThumbnail(
        BUF_CONVERT_INFO* src,
        MISC_FIELD_INFO* info,
        uint8_t* pThumbBuf,
        int32_t bufSize,
        int32_t* pByteUsed);

    int32_t createThumbnailJpeg(
        uint32_t srcPhyAddr,
        uint32_t srcWidth,
        uint32_t srcHeight,
        int32_t thumbWidth,
        int32_t thumbHeight);

    void getBufferVAddr(
        buffer_handle_t* buffer,
        int format,
        int w,
        int h,
        void** ppVPtr);

    void getBufferCbCr(
        buffer_handle_t* buffer,
        int w,
        int h,
        void** ppVPtr,
        void** ppCb,
        void** ppCr);

    void getBufferPhyAddr(
        buffer_handle_t* buffer,
        uint32_t* phyAddr);

    void dumpData(void* vaddr,
                  int size,
                  char* path,
                  uint32_t* pCnt);

    void setRtkPrivData(
        StreamBuffer* pStreamBuf,
        uint32_t inputFps);

};

}; // namespace androidcamera3


#endif
